from enum import Enum
from typing import Any, Iterable, Iterator, KeysView, Mapping, Optional, Union


class Color(Enum):
    BLACK = "black"
    RED = "red"
    GREEN = "green"
    YELLOW = "yellow"
    BLUE = "blue"
    MAGENTA = "magenta"
    CYAN = "cyan"
    WHITE = "white"

    def __str__(self):
        return self.value


class Style(Enum):
    NONE = "none"
    BOLD = "bold"
    FAINT = "faint"
    ITALIC = "italic"
    UNDERLINE = "underline"
    BLINK = "blink"
    BLINK2 = "blink2"
    NEGATIVE = "negative"
    CONCEALED = "concealed"
    CROSSED = "crossed"

    def __str__(self):
        return self.value


class Format(Mapping[str, Any]):
    """
    This class represents a format for a log message. It can be used to specify the foreground and background colors and other ANSI styles.

    Args:
        foreground: The foreground color.
        background: The background color.
        style: The style. This can be a single style or several styles.
    """

    def __init__(
        self,
        foreground: Color = None,
        background: Color = None,
        style: Union[Style, Iterable[Style]] = (),
    ):
        self._dict = {
            "fg": str(foreground) if foreground else None,
            "bg": str(background) if background else None,
            "style": str(style)
            if isinstance(style, Style)
            else "+".join(str(s) for s in style),
        }

    def keys(self) -> KeysView[str]:
        return self._dict.keys()

    def __getitem__(self, key: str) -> Optional[str]:
        return self._dict[key]

    def __iter__(self) -> Iterator[str]:
        return iter(self._dict)

    def __len__(self) -> int:
        return len(self._dict)



"""
.. include:: ../README.md
"""


try:
    import importlib.metadata as importlib_metadata
except ImportError:
    import importlib_metadata as importlib_metadata  # type: ignore
try:
    __version__ = importlib_metadata.version(__package__ or __name__)
except importlib_metadata.PackageNotFoundError:
    import toml

    __version__ = (
        toml.load("pyproject.toml")
        .get("tool", {})
        .get("poetry", {})
        .get("version", "unknown")
        + "-dev"
    )

from .formatter import Formatter
from .ansi import Format, Color, Style



import logging
from typing import Any, Mapping, Union
from colors import color
from .ansi import Color, Format, Style


default_config = {
    logging.DEBUG: Format(style=Style.FAINT),
    logging.INFO: Format(foreground=Color.GREEN),
    logging.WARNING: Format(foreground=Color.YELLOW),
    logging.ERROR: Format(foreground=Color.RED),
    logging.CRITICAL: Format(foreground=Color.RED, style=[Style.BOLD, Style.UNDERLINE]),
}

default_exception_config = default_config[logging.CRITICAL]
default_stack_config = default_config[logging.ERROR]


class Formatter(logging.Formatter):
    """This log formatter wraps a given formatter and adds color to the output.

    If you want to use different colors from the default ones, you can pass a mapping to the color_configs argument in the constructor. This mapping should be from log levels (ints) to Format objects.

    Note: Instead of Format objects, you can pass any mapping acceptable as keyword arguments for the ansicolors library's color function. You can find more information on this library here: https://github.com/jonathaneunice/colors.
    """

    def __init__(
        self,
        inner_formatter: logging.Formatter,
        color_configs: Mapping[int, Mapping[str, Any]] = default_config,
        exception_config: Mapping[str, Any] = default_exception_config,
        stack_config: Mapping[str, Any] = default_stack_config,
    ):
        """
        Args:
            inner_formatter: The formatter to use for the log messages.
            color_configs: A mapping from log levels to Format objects.
            exception_config: The Format for exceptions.
            stack_config: The Format for stack traces.
        """
        self.formatter = inner_formatter
        self.configs = color_configs
        self.exception_config = exception_config
        self.stack_config = stack_config

    def format(self, record: logging.LogRecord) -> str:
        return color(
            self.formatter.format(record), **self.configs.get(record.levelno, {})
        )

    def formatException(self, exc_info):
        return color(self.formatter.formatException(exc_info), **self.exception_config)

    def formatStack(self, stack_info):
        return color(self.formatter.formatStack(stack_info), **self.stack_config)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['rainbowlog']

package_data = \
{'': ['*']}

install_requires = \
['ansicolors>=1.1,<2.0', 'importlib-metadata>=4.11,<5.0']

setup_kwargs = {
    'name': 'rainbowlog',
    'version': '1.0.0',
    'description': 'Format your python logs with colours based on the log levels.',
    'long_description': '# Rainbow Log\n\nFormat your python logs with colours based on the log levels.\n\n## Installation\n\n\tpip install rainbowlog\n\n## Docs\n\nYou can find the documentation [here](https://abrahammurciano.github.io/rainbowlog/rainbowlog)\n\n## Usage\n\nHere\'s a basic example of a script that logs colorfully to the console, but regularly to a file.\n\n```python\nimport logging\nimport rainbowlog\n\nlogger = logging.getLogger(__name__)\n\n# This one will write to the console\nstream_handler = logging.StreamHandler()\n\n# This one will write to a file\nfile_handler = logging.FileHandler("output.log")\n\n# Here we decide how we want the logs to look like\nformatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")\n\n# We want the stream handler to be colorful\nstream_handler.setFormatter(rainbowlog.Formatter(formatter))\n\n# We don\'t want the file handler to be colorful\nfile_handler.setFormatter(formatter)\n\n# Finally we add the handlers to the logger\nlogger.addHandler(stream_handler)\nlogger.addHandler(file_handler)\n\nif __name__ == "__main__":\n\tlogger.debug("This is a debug message")\n\tlogger.info("This is an info message")\n\tlogger.warning("This is a warning message")\n\tlogger.error("This is an error message")\n\tlogger.critical("This is a critical message")\n```\n\nIf you want to change the format of the logs for each log level, you can construct the `rainbowlog.Formatter` object like this:\n\n```py\nimport logging\nfrom rainbowlog import Formatter, Format, Color, Style\n\nformatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")\ncolor_formatter = Formatter(\n\tformatter,\n\tcolor_configs={\n\t\tlogging.DEBUG: Format(Color.BLUE, style=Style.FAINT),\n\t\tlogging.INFO: Format(Color.GREEN),\n\t\tlogging.WARNING: Format(Color.YELLOW, style=Style.ITALIC),\n\t\tlogging.ERROR: Format(Color.RED, Color.WHITE, Style.BOLD),\n\t\tlogging.CRITICAL: Format(Color.RED, Color.YELLOW, (Style.BOLD, Style.UNDERLINE)),\n\t}\n\texception_config=Format(Color.RED, Color.WHITE, Style.BOLD),\n\tstack_config=Format(Color.RED, Color.WHITE, Style.BOLD),\n)\n```\n\n> NOTE: You can pass instead of a Format object, a dict of keyword arguments which ansicolors library\'s `color` function accepts. See the [ansicolors documentation](https://pypi.org/project/ansicolors/). This will usually not be necessary.',
    'author': 'Abraham Murciano',
    'author_email': 'abrahammurciano@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/abrahammurciano/rainbowlog',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
